/**
 * Copyright (C), 2015-2019
 * Author:   HJL
 * Date:     2019/7/19 14:39
 * Description:病案召回
 */
package com.emr.controller;

import com.emr.annotation.OptionalLog;
import com.emr.entity.*;
import com.emr.service.Archive_MasterService;
import com.emr.service.Emr_DictionaryService;
import com.emr.service.Emr_Fault_DetailService;
import com.emr.service.LogService;
import com.emr.util.ExportExcelUtil;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.session.Session;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

@PropertySource(value = "classpath:config/jdbc.properties", encoding = "UTF-8")
@Controller
@RequestMapping("/medicalRecall")
public class MedicalRecallController {
    @Autowired
    private Emr_Fault_DetailService emrFaultDetailService;

    @Autowired
    private Emr_DictionaryService emrDictionaryService;

    @Autowired
    private Archive_MasterService archiveMasterService;

    @Value("${recallReason}")
    private String recallReason;

    @RequestMapping(value = "/recall")
    public String recall(Model model) {
        return "medicalRecallDir/medicalRecallList";
    }


    @ResponseBody
    @RequestMapping(value = "/recallList")
    public OffsetLimitPage recallList(Emr_Fault_Vo emrFaultVo, Integer offset, Integer limit) {
        OffsetLimitPage result = emrFaultDetailService.selectByCol(emrFaultVo, offset, limit);
        return result;
    }

    @ResponseBody
    @RequestMapping(value = "/exportExcel")
    public void exportExcel(HttpServletResponse response, Emr_Fault_Vo emrFaultVo) throws Exception {
        String tableThNames = "住院号,住院次数,名字,性别,身份证,入院科室,入院日期,出院科室,出院日期,状态,归档状态";
        String fieldCns = "inpNo,visitId,name,sex,idNo,deptAdmissionTo,admissionDateTime,deptName,dischargeDateTime,state,archivestate";
        //构造excel的数据
        List<Emr_Fault_Vo> list = emrFaultDetailService.selectByCol(emrFaultVo);

        Emr_Dictionary dic = new Emr_Dictionary();
        dic.setEffective(1);
        dic.setTypecode("dept_code");
        //科室列表
        List<Emr_Dictionary> dicList = emrDictionaryService.dicByTypeCode(dic);
        for (int i = 0; i < list.size(); i++) {
            //替换科室
            for (int k = 0; k < dicList.size(); k++) {
                String deptName = list.get(i).getDeptName();
                if (deptName.equals(dicList.get(k).getCode())) {
                    //出院科室
                    deptName = deptName.replace(deptName, dicList.get(k).getName());
                    list.get(i).setDeptName(deptName);
                    //入院科室dept_admission_to
                    deptName = list.get(i).getDeptAdmissionTo();
                    deptName = deptName.replace(deptName, dicList.get(k).getName());
                    list.get(i).setDeptAdmissionTo(deptName);
                }
            }
        }

        //文件名
        String fileName = "召回信息数据" + new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()) + ".xls";
        //ExportExcelUtil
        ExportExcelUtil exportExcelUtil = new ExportExcelUtil();
        response.setContentType("application/ms-excel;charset=gbk");
        //导出excel的操作
        exportExcelUtil.expordExcel(tableThNames, fieldCns, list, fileName, response);
    }

    @ResponseBody
    @RequestMapping(value = "/updateState")
    public int updateState(HttpServletRequest request, HttpServletResponse response, Emr_Fault_Detail emrFaultDetail) {
        try{
            Subject currentUser = SecurityUtils.getSubject();
            Session session = currentUser.getSession();
            String username = (String) session.getAttribute("userSession");//获取前面登录的用户名
            emrFaultDetail.setUpdater(username);
            SimpleDateFormat from1=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            emrFaultDetail.setUpdateTime(from1.format(new Date()));
            if(emrFaultDetail.getRecallReason()==null ||emrFaultDetail.getRecallReason()==""){
                emrFaultDetail.setRecallReason(recallReason);
            }
            //emrFaultDetail.setRecallReason("医院需要这份病案");
            Archive_Master archiveMaster=new Archive_Master();
            archiveMaster.setId(emrFaultDetail.getArchiveDetailId());
            archiveMaster.setArchivestate("初审");
            int bol=archiveMasterService.updateByClo(archiveMaster);
            emrFaultDetail.setArchiveDetailId(null);
            bol = emrFaultDetailService.updateCloByPrimaryKey(emrFaultDetail);
            return 1;
        }catch (Exception e){
            e.printStackTrace();
            return 0;
        }
    }
}
 

