/**
 * Copyright (C), 2015-2019
 * Author:   HJL
 * Date:     2019/7/19 15:39
 * Description:患者住院信息主索引管理
 */
package com.emr.service.ipml;

import com.emr.dao.Archive_MasterMapper;
import com.emr.dao.CompenSateMapper;
import com.emr.dao1.OracleMapper;
import com.emr.entity.*;
import com.emr.service.Archive_MasterService;
import com.emr.service.Emr_DictionaryService;
import com.emr.util.ExceptionPrintUtil;
import com.emr.util.ListUtil;
import com.emr.util.Setters;
import com.emr.vo.CompenSateVo;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections4.ListUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class Archive_MasterServiceImpl implements Archive_MasterService {

    @Autowired
    private Archive_MasterMapper archiveMasterMapper;

    @Autowired
    private Emr_DictionaryService emrDictionaryService;

    @Autowired
    private OracleMapper oracleMapper;
    @Autowired
    private CompenSateMapper compenSateMapper;
    @Override
    public OffsetLimitPage selectByCol(Archive_Master_Vo archiveMasterVo, Integer offset, Integer limit) {
        PageHelper.offsetPage(offset, limit);
        List<Archive_Master_Vo> list = selectByCol(archiveMasterVo);
        return new OffsetLimitPage((Page) list);
    }

    @Override
    public List<Archive_Master_Vo> selectByCol(Archive_Master_Vo archiveMasterVo) {
        List<Archive_Master_Vo> list= archiveMasterMapper.selectByCol(archiveMasterVo);
        //设置科室
        setDeptNameForList(list);
        return list;
    }

    @Override
    public List<Archive_Master_Vo> selectByColumn(Archive_Master_Vo archiveMasterVo) {
        List<Archive_Master_Vo> list = archiveMasterMapper.selectByColumn(archiveMasterVo);
        //设置科室
        setDeptNameForList(list);
        return list;
    }

    @Override
    public List<Archive_Master_Vo> selectByUnfile(Archive_Master_Vo archiveMasterVo) {
        return archiveMasterMapper.selectByUnfile(archiveMasterVo);
    }

    @Override
    public List<Archive_Master_Vo> selectByLast(Archive_Master_Vo archiveMasterVo) {

        List<Archive_Master_Vo> list = archiveMasterMapper.selectByLast(archiveMasterVo);
        //设置科室
        setDeptNameForList(list);
        return list;
    }

    @Override
    public OffsetLimitPage selectByColumn(Archive_Master_Vo archiveMasterVo, Integer offset, Integer limit) {
        PageHelper.offsetPage(offset, limit);
        List<Archive_Master_Vo> list = selectByColumn(archiveMasterVo);
        return new OffsetLimitPage((Page) list);
    }

    //设置科室
    private void setDeptNameForList(List<Archive_Master_Vo> list) {
        if(!CollectionUtils.isEmpty(list)) {
            //查询科室集合
            Emr_Dictionary dictionary = new Emr_Dictionary();
            dictionary.setTypecode("dept_code");
            List<Emr_Dictionary> deptList = emrDictionaryService.dicByTypeCode(dictionary);
            for (Archive_Master_Vo master : list) {
                //设置科室
                if (!CollectionUtils.isEmpty(deptList)) {
                    for (Emr_Dictionary dictionary1 : deptList) {
                        //出院科室
                        if (master.getDeptName().equals(dictionary1.getCode())) {
                            master.setDeptName(dictionary1.getName());
                        }
                        //入院科室
                        if (master.getDeptAdmissionTo().equals(dictionary1.getCode())) {
                            master.setDeptAdmissionTo(dictionary1.getName());
                        }
                    }
                }
            }
        }
    }

    @Override
    public OffsetLimitPage selectByUnfile(Archive_Master_Vo archiveMasterVo, Integer offset, Integer limit) {
        PageHelper.offsetPage(offset, limit);
        List<Archive_Master_Vo> list = archiveMasterMapper.selectByUnfile(archiveMasterVo);
        return new OffsetLimitPage((Page) list);
    }

    @Override
    public OffsetLimitPage selectByLast(Archive_Master_Vo archiveMasterVo, Integer offset, Integer limit) {
        PageHelper.offsetPage(offset, limit);
        List<Archive_Master_Vo> list = archiveMasterMapper.selectByLast(archiveMasterVo);
        //设置科室设置纸质签收状态和his召回状态
        setPaperStatusAndCallbackStatus(list);
        if (list.size()>0){
            //按需采集
            //masterIds集合
            List<String> masterIds = ListUtil.distinctSelect(list, Archive_Master_Vo::getId);
            //分批次
            List<List<String>> newList = ListUtils.partition(masterIds, 1000);
            for (List<String> messageList : newList) {
                //转成逗号拼接
                String ids = String.join(",", messageList);
                //获取按需采集信息
                List<CompenSateVo> hisAll = compenSateMapper.selectAll(ids);
                Map<String, CompenSateVo> masterIdHisMap = ListUtil.toMap(hisAll, CompenSateVo::getMasterId);
                Setters.<Archive_Master_Vo>instance().list(list).cycleSetProperties(p -> {
                    String masterId = p.getId();
                    if (masterIdHisMap.containsKey(masterId)){
                        p.setCompensateState(masterIdHisMap.get(masterId).getCompensateState());
                    }else {
                        p.setCompensateState(1);
                    }
                });
                List<CompenSateVo> pacsAll = compenSateMapper.selectAllByPacs(ids);
                Map<String, CompenSateVo> masterIdPacsMap = ListUtil.toMap(pacsAll, CompenSateVo::getMasterId);
                Setters.<Archive_Master_Vo>instance().list(list).cycleSetProperties(p -> {
                    String masterId = p.getId();
                    if (masterIdPacsMap.containsKey(masterId)){
                        p.setPacsCompensate(masterIdPacsMap.get(masterId).getCompensateState());
                    }else {
                        p.setPacsCompensate(1);
                    }
                });
            }
        }

        return new OffsetLimitPage((Page) list);
    }

    /**
     * 终审设置纸质签收状态和his召回状态
     * @param list
     */
    @Override
    public void setPaperStatusAndCallbackStatus(List<Archive_Master_Vo> list) {
        if(!CollectionUtils.isEmpty(list)){
            StringBuilder patientIdsBuilder = new StringBuilder();
            for(Archive_Master_Vo master : list){
                if(StringUtils.isNotBlank(patientIdsBuilder)){
                    patientIdsBuilder.append(",");
                }
                patientIdsBuilder.append("'").append(master.getPatientId()).append("'");
            }
            List<OracleEntity> paperPatiendIds = new ArrayList<>();
            List<OracleEntity> callBackPatiendIds = new ArrayList<>();
//            try {
//                //查询纸质签收和召回状态
//                paperPatiendIds = this.selectPaperPatiendIdList(patientIdsBuilder.toString());
//                callBackPatiendIds = this.selectCallBackPatiendIdList(patientIdsBuilder.toString());
//            }catch (Exception e){
//                e.printStackTrace();
//                ExceptionPrintUtil.printException(e);
//            }
            //查询科室集合
            Emr_Dictionary dictionary = new Emr_Dictionary();
            dictionary.setTypecode("dept_code");
            List<Emr_Dictionary> deptList = emrDictionaryService.dicByTypeCode(dictionary);
            for(Archive_Master_Vo master : list){
                //设置科室
                if(!CollectionUtils.isEmpty(deptList)){
                    for(Emr_Dictionary dictionary1 : deptList){
                        //出院科室
                        if(master.getDeptName().equals(dictionary1.getCode())){
                            master.setDeptName(dictionary1.getName());
                        }
                        //入院科室
                        if(master.getDeptAdmissionTo().equals(dictionary1.getCode())){
                            master.setDeptAdmissionTo(dictionary1.getName());
                        }
                    }
                }
                String patientId = master.getPatientId();
                master.setPaperStatus("未签收");
                master.setCallBackStatus("未召回");
                //设置纸质签收状态
                if(!CollectionUtils.isEmpty(paperPatiendIds)){
                    for(OracleEntity paperPatientId : paperPatiendIds){
                        if(paperPatientId.getFpatno().equals(patientId)){
                            master.setPaperStatus("已签收");
                            break;
                        }
                    }
                }
                //设置召回状态
                if(!CollectionUtils.isEmpty(callBackPatiendIds)){
                    for(OracleEntity callBackPatientId : callBackPatiendIds){
                        if(callBackPatientId.getFpatno().equals(patientId)){
                            master.setCallBackStatus("已召回");
                            break;
                        }
                    }
                }
                //设置修改状态
                if(StringUtils.isNotBlank(master.getUpdateStatus())){
                    master.setUpdateStatus("已修改");
                }else{
                    master.setUpdateStatus("未修改");
                }
            }
        }
    }

    @Override
    public List<Archive_Master> selectByObject(Archive_Master master) {
        return archiveMasterMapper.selectByObject(master);
    }

    /**
     *获取查询his oracle数据库的纸质签收信息
     * @param patientIds 逗号隔开的记账号集合字符串
     * @return
     */
    @Override
    public List<OracleEntity> selectPaperPatiendIdList(String patientIds){
        return oracleMapper.selectPaperPatiendIdList(patientIds);
    }

    /**
     *获取查询his oracle数据库的召回信息
     * @param patientIds 逗号隔开的记账号集合字符串
     * @return
     */
    @Override
    public List<OracleEntity> selectCallBackPatiendIdList(String patientIds){
        return oracleMapper.selectCallBackPatiendIdList(patientIds);
    }

    @Override
    public int updateByClo(Archive_Master record) {
        return archiveMasterMapper.updateByClo(record);
    }

    @Override
    public int updateById(Archive_Master record) {
        return archiveMasterMapper.updateById(record);
    }

    @Override
    public Archive_Master selectByPrimaryKey(String id) {
        return archiveMasterMapper.selectByPrimaryKey(id);
    }
}
 

