package com.emr.service.ipml;

import com.alibaba.fastjson.JSON;
import com.emr.dao.StatisticsMapper;
import com.emr.dao.TUuInfoMapper;
import com.emr.dao.TUuPrintMapper;
import com.emr.entity.*;
import com.emr.service.Emr_DictionaryService;
import com.emr.vo.*;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import net.sf.json.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2020/1/8 15:54
 * @UpdateUser: 曾文和
 * @UpdateDate: 2020/1/8 15:54
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@Service
@Transactional
public class StatisticsService {
    @Value("${POWER_URLHEAD}")
    private String POWER_URLHEAD;
    @Autowired
    private StatisticsMapper statisticsMapper;
    @Autowired
    private TUuPrintMapper uuPrintMapper;
    @Autowired
    private TUuInfoMapper uuInfoMapper;
    @Autowired
    private Emr_DictionaryService emrDictionaryService;
    //终审按天统计
    public List<FinalAndFirstStatistics> getFinalStatistics(HttpServletRequest request,Integer offset, Integer limit, String startDate, String endDate,String sql) {
        if(null != offset && null != limit){
            PageHelper.offsetPage(offset, limit);
        }
        List<FinalAndFirstStatistics> list = statisticsMapper.finalStatistics(startDate, endDate,sql);
        //转换姓名
        List<User> userList = getUserList(request);
        for(FinalAndFirstStatistics statistics:list) {
            for (User user : userList) {
                if(StringUtils.isNotBlank(statistics.getCheckCode()) && statistics.getCheckCode().equals(user.getUserName())){
                    statistics.setCheckName(user.getName());
                    break;
                }
            }
        }
        return list;
    }


    //初审按天统计
    public List<FinalAndFirstStatistics> getFirstStatistics(HttpServletRequest request, Integer offset, Integer limit, String startDate, String endDate,String sql) {
        if(null != offset && null != limit){
            PageHelper.offsetPage(offset, limit);
        }
        List<FinalAndFirstStatistics> list = statisticsMapper.firstStatistics(startDate, endDate,sql);
        //转换姓名
        List<User> userList = getUserList(request);
        for(FinalAndFirstStatistics statistics:list) {
            for (User user : userList) {
                if(statistics.getCheckCode().equals(user.getUserName())){
                    statistics.setCheckName(user.getName());
                    break;
                }
            }
        }
        return list;
    }

    public OffsetLimitPage getStatisticsDetail(HttpServletRequest request, Integer offset, Integer limit,String disStartDate, String disEndDate, Archive_Master_Vo archiveMasterVo,Integer flag) {
        if(null != offset && null != limit){
            PageHelper.offsetPage(offset, limit);
        }
        List<Archive_Master_Vo> list = getDetailList(request,disStartDate, disEndDate, archiveMasterVo, flag,null);
        return new OffsetLimitPage((Page) list);
    }

    public List<Archive_Master_Vo> getDetailList(HttpServletRequest request,String disStartDate, String disEndDate, Archive_Master_Vo archiveMasterVo,Integer flag,String sql){
        //转换姓名
        List<User> userList = getUserList(request);
        String checkDoctor = archiveMasterVo.getCheckDoctor();
        //模糊搜索审核姓名转换工号
        if(StringUtils.isNoneBlank(checkDoctor)){
            StringBuilder checkNames = new StringBuilder();
            for(User user:userList){
                if(StringUtils.isNoneBlank(user.getName()) && user.getName().contains(checkDoctor)){
                    if(StringUtils.isNotBlank(checkNames)){
                        checkNames.append(",");
                    }
                    checkNames.append("'").append(user.getUserName()).append("'");
                }
            }
            archiveMasterVo.setCheckDoctor(checkNames.toString());
        }
        List<Archive_Master_Vo> list = statisticsMapper.statistics(disStartDate,disEndDate,archiveMasterVo,flag,sql);
        //查询科室集合
        Emr_Dictionary dictionary = new Emr_Dictionary();
        dictionary.setTypecode("dept_code");
        List<Emr_Dictionary> deptList = emrDictionaryService.dicByTypeCode(dictionary);
        for(Archive_Master_Vo statistics:list) {
            //设置科室
            if(!CollectionUtils.isEmpty(deptList)){
                for(Emr_Dictionary dictionary1 : deptList){
                    //出院科室
                    if(statistics.getDeptName().equals(dictionary1.getCode())){
                        statistics.setDeptName(dictionary1.getName());
                    }
                    //入院科室
                    if(statistics.getDeptAdmissionTo().equals(dictionary1.getCode())){
                        statistics.setDeptAdmissionTo(dictionary1.getName());
                    }
                }
            }
            for (User user : userList) {
                if(StringUtils.isNotBlank(statistics.getCheckDoctor()) && statistics.getCheckDoctor().equals(user.getUserName())){
                    statistics.setCheckName(user.getName());
                }
                if(StringUtils.isNotBlank(statistics.getCheckedDoctor()) && statistics.getCheckedDoctor().equals(user.getUserName())){
                    statistics.setCheckedName(user.getName());
                }
            }
        }
        return list;
    }

    //复印记录报表
    public List<TUuPrintVo> getPrintCount(TUuPrintSearch search){
        List<TUuPrintVo> list = uuPrintMapper.getPrintCount(search);
        return list;
    }

    //复印记录明细
    public List<TUuPrintVo> getPrintInfo(TUuPrintSearch search){
        List<TUuPrintVo> list = uuPrintMapper.getPrintInfo(search);
        if(null != list && !list.isEmpty()){
            SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            for(TUuPrintVo printVo : list){
                Date cpytime = printVo.getCpytime();
                printVo.setPrintTime(fmt.format(cpytime));
            }
        }
        return list;
    }

    //扫描记录报表
    public List<TUuInfoVo> getScanCount(TUuPrintSearch search){
        List<TUuInfoVo> list = uuInfoMapper.getScanCount(search);
        return list;
    }

    //扫描记录明细
    public List<TUuInfoVo> getScanInfo(TUuPrintSearch search){
        List<TUuInfoVo> list = uuInfoMapper.getScanInfo(search);
        return list;
    }

    public  List<User> getUserList(HttpServletRequest request){
        List<User> userList = new ArrayList<>();
        try {
            //查询缓存时候存在
            userList = (List<User>)request.getSession().getAttribute("USER_LIST");
            if(null == userList || userList.isEmpty()){
                String userName = (String) request.getSession().getAttribute("userSession");
                //调用接口查询
                String resultString = "";
                // 创建uri
                String url = POWER_URLHEAD+"/font/getUserList?userName="+userName;
                // 执行请求
                CloseableHttpResponse response = HttpClients.createDefault().execute(new HttpGet(url));
                // 判断返回状态是否为200
                if (response.getStatusLine().getStatusCode() == 200) {
                    resultString = EntityUtils.toString(response.getEntity(), "UTF-8");
                }
                JSONObject jsonObject = JSONObject.fromObject(resultString);
                String extend = jsonObject.getString("extend");
                JSONObject extendObject = JSONObject.fromObject(extend);
                String userList1 = extendObject.getString("userList");
                userList = (List<User>) JSON.parseArray(userList1,User.class);
                //设置进session
                request.getSession().setAttribute("USER_LIST",userList);
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return userList;
    }

    public HttpServletRequest getRequest(){
        return ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
    }

    //获取当前用户
    public Power_User getCurrentUser(){
        return (Power_User)getRequest().getSession().getAttribute("CURRENT_USER");
    }

    //获取当前用户名
    public String getCurrentUserName(){
        return getCurrentUser().getUserName();
    }
}

