package com.emr.util;

import com.lowagie.text.Document;
import com.lowagie.text.Element;
import com.lowagie.text.Image;
import com.lowagie.text.Utilities;
import com.lowagie.text.pdf.*;
import com.lowagie.text.pdf.codec.TiffImage;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServletResponse;
import java.awt.*;
import java.io.*;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2019/6/6 9:07
 * @UpdateUser: 曾文和
 * @UpdateDate: 2019/6/6 9:07
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
public class Jpg2PdfUtil {
    static Logger  logger = LoggerFactory.getLogger(Jpg2PdfUtil.class);
    public static void imageToPdf(HttpServletResponse response, List<String> filePaths, String pdfName) throws Exception{
        // 实例化图牿
        Image image = null;
        pdfName = java.net.URLEncoder.encode(pdfName, "UTF-8");
        Document doc = openDocument(response, pdfName);
        try {
            for (String filePath : filePaths) {
                File file1 = new File(filePath);
                if(file1.exists()){
                    if (file1.getName().endsWith(".tif") || file1.getName().endsWith(".tiff")) {
                        Object localObject1 = null;
                        Object localObject2 = null;
                        URL paramURL = Utilities.toURL(filePath);
                        try {
                            if (paramURL.getProtocol().equals("file")) {
                                localObject2 = paramURL.getFile();
                                localObject2 = Utilities
                                        .unEscapeURL((String) localObject2);
                                localObject1 = new RandomAccessFileOrArray(
                                        (String) localObject2);
                            } else {
                                localObject1 = new RandomAccessFileOrArray(paramURL);
                            }

                            int pageNums = TiffImage
                                    .getNumberOfPages((RandomAccessFileOrArray) localObject1);
                            if (pageNums > 0) {
                                for (int i = 1; i <= pageNums; i++) {
                                    localObject2 = TiffImage.getTiffImage(
                                            (RandomAccessFileOrArray) localObject1, i);
                                    image = (Image) localObject2;
                                }
                            }
                            if (localObject1 != null)
                                ((RandomAccessFileOrArray) localObject1).close();

                        } finally {
                            if (localObject1 != null)
                                ((RandomAccessFileOrArray) localObject1).close();
                        }
                    }else if (file1.getName().endsWith(".png")
                            || file1.getName().endsWith(".jpg")
                            || file1.getName().endsWith(".gif")
                            || file1.getName().endsWith(".jpeg")
                    ) {
                        // 实例化图牿
                        image = Image.getInstance(filePath);
                    }
                    // 获得图片的高庿
                    float heigth = image.getHeight();
                    float width = image.getWidth();
                    // 合理压缩，h>w，按w压缩，否则按w压缩
                    int percent = getPercent(heigth, width);

                    // 设置图片居中显示
                    //image.setAlignment(Image.MIDDLE);
                    image.setAlignment(Image.ALIGN_CENTER);
                    //image.scaleAbsolute(500, 400);
                    // 按百分比显示图片的比便
                    if (width > 1024 || heigth > 786) {
                        image.scalePercent(percent+5);
                    }
                    doc.add(image);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (doc.isOpen()) {
                doc.close();
            }
        }
    }

    private static Document openDocument(HttpServletResponse response,String pdfName) throws Exception{
        Document doc = new Document();
        response.reset();
        response.setCharacterEncoding("utf-8");
        response.setContentType("application/pdf"); // word格式
        response.setHeader("Content-Disposition", "attachment; filename=" + pdfName+ "("+ new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()) +").pdf");
        // 定义输出文件的位罿
        PdfWriter.getInstance(doc,response.getOutputStream());
        // 开启文桿
        doc.open();
        return doc;
    }

    /**
     * 利用itext打开pdf文档
     */
    public static boolean check(String file) {
        boolean flag1 = false;
        int n = 0;
        Document document = null;
        PdfReader pdfReader = null;
        try {
            File f = new File(file);
            if(f.isFile()){
                pdfReader = new PdfReader(file);
                document = new Document(pdfReader.getPageSize(1));
                document.open();
                n = pdfReader.getNumberOfPages();
                if (n != 0) {
                    flag1 = true;
                }
            }
        } catch (Exception e) {
        }finally {
            if(null != pdfReader){
                pdfReader.close();
            }
            if(null != document){
                document.close();
            }
        }
        return flag1;
    }

    public static void mulFile2One(HttpServletResponse response,List<String> files,String waterMarkName) {
        SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //遍历删除,除去损坏，文件不存在，抛异常就是空白页
        Iterator<String> iterator = files.iterator();
        while (iterator.hasNext()) {
            String fileStr = iterator.next();
            boolean flag = false;
            flag = check(fileStr);
            if(!flag){
                iterator.remove();
                System.out.println(fmt.format(new Date())+":"+fileStr+"文件损坏");
            }
        }
        if(null != files && !files.isEmpty()){
            // pdf合并工具类
            Document document = null;
            ByteArrayOutputStream bos = null;
            PdfCopy copy = null;
            PdfReader reader = null;
            PdfReader pdfReader = null;
            try {
                response.reset();
                document = new Document(new PdfReader(files.get(0)).getPageSize(1));
                copy = new PdfCopy(document,response.getOutputStream());
                document.open();
                bos = new ByteArrayOutputStream();
                for (int i = 0; i < files.size(); i++) {
                    bos.flush();
                    //判断是否加水印
                    if(StringUtils.isNotBlank(waterMarkName)){
                        setWatermark(bos,files.get(i), waterMarkName, null);
                    }
                    reader = new PdfReader(files.get(i));
                    int n = reader.getNumberOfPages();
                    pdfReader = new PdfReader(bos.toByteArray());
                    for (int j = 1; j <= n; j++) {
                        document.newPage();
                        PdfImportedPage page = null;
                        if(StringUtils.isNotBlank(waterMarkName)){
                            page = copy.getImportedPage(pdfReader, j);
                        }else{
                            page = copy.getImportedPage(reader, j);
                        }
                        copy.addPage(page);
                    }
                }
            }catch (Exception e){
                e.printStackTrace();
            }finally {
                try{
                    if(null != copy){
                        copy.close();
                    }
                    if(pdfReader != null){
                        pdfReader.close();
                    }
                    if(null != reader){
                        reader.close();
                    }
                    if(null != bos){
                        bos.flush();
                        bos.close();
                    }
                    if(null != document){
                        document.close();
                    }
                }catch (Exception e){
                    e.printStackTrace();
                }
            }
        }
    }


    //加水印
    private static void setWatermark(ByteArrayOutputStream bos, String input, String waterMarkName, String imgPath)
            throws Exception{
        PdfReader reader = new PdfReader(input);
        PdfStamper stamper = new PdfStamper(reader, bos);
        int total = reader.getNumberOfPages() + 1;
        PdfContentByte content;
        BaseFont base = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.EMBEDDED);
        PdfGState gs = new PdfGState();
        for (int i = 1; i < total; i++) {
            content = stamper.getOverContent(i);// 在内容上方加水印
            //加文字水印
            if(StringUtils.isNotBlank(waterMarkName)) {

                gs.setFillOpacity(0.3f);
                gs.setStrokeOpacity(0.3f);
                content.beginText();
                content.setColorFill(Color.LIGHT_GRAY);
                content.setFontAndSize(base, 50);
                content.setTextMatrix(70, 200);
                //v:距左 v1:距下 v2:
                content.showTextAligned(Element.ALIGN_CENTER, waterMarkName, 300, 600, 40);
                content.showTextAligned(Element.ALIGN_CENTER, waterMarkName, 300, 200, 40);
                content.showTextAligned(Element.ALIGN_CENTER, waterMarkName, 800, 600, 40);
                content.showTextAligned(Element.ALIGN_CENTER, waterMarkName, 800, 200, 40);
                content.showTextAligned(Element.ALIGN_CENTER, waterMarkName, 300, 1100, 40);
                content.showTextAligned(Element.ALIGN_CENTER, waterMarkName, 300, 1500, 40);
                content.showTextAligned(Element.ALIGN_CENTER, waterMarkName, 800, 1100, 40);
                content.showTextAligned(Element.ALIGN_CENTER, waterMarkName, 800, 1500, 40);
                content.endText();
            }
            if(StringUtils.isNotBlank(imgPath)){
                Image image = Image.getInstance(imgPath);
                image.setAbsolutePosition(200, 206); // set the first background
                image.scaleToFit(200, 200);
                content.addImage(image);
            }
            content.setColorFill(Color.BLACK);
            content.setFontAndSize(base, 8);
        }
        stamper.close();
        if(null != reader){
            reader.close();
        }
    }
    /**
     * 在不改变图片形状的同时，判断，如果h>w，则按h压缩，否则在w>h或w=h的情况下，按宽度压缩
     * @param h
     * @param w
     * @return
     */
    public static int getPercent(float h, float w) {
        int p = 0;
        float p2 = 0.0f;
        if (h > w) {
            p2 = 210 / h * 279;
        } else {
            p2 = 210 / w * 279;
        }
        p = Math.round(p2);
        return p;
    }

}
